{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Update Share Group Type" %}{% endblock %}

{% block main %}
    {% include 'admin/share_group_types/_update.html' %}
{% endblock %}
