{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}
{% block modal-body-right %}
  <h3>{% trans "Description" %}:</h3>
  <p>{% trans "Create share replica in specific availability zone." %}
{% endblock %}
