{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}
{% block modal-body-right %}
  <h3>{% trans "Description" %}:</h3>
  <p>{% blocktrans %}
    Share networks contain network data, that will be used
    for creation of service VM, where will be hosted shares.
    {% endblocktrans %}</p>
{% endblock %}
